#
#  Copyright 1999-2002 Illinois Institute of Technology
#
#  See the file "LICENSE" for information on usage and redistribution
#  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

proc setup_imcagui_callbacks { record_list } {

	find_callback_record $record_list "lambda"       "lambda"
	find_callback_record $record_list "d_spacing"    "d_spacing"

	if { $IMCAGui::show_slit_positions } {
		find_callback_record $record_list "hslit_center" "bh_center"
		find_callback_record $record_list "vslit_center" "bv_center"
	}

	find_callback_record $record_list "hslit_size"   "bh_size"
	find_callback_record $record_list "vslit_size"   "bv_size"
	find_callback_record $record_list "edge_energy"  "edge_energy"

	if { $IMCAGui::have_undulator } {
	    find_callback_record $record_list "id_ev"          "id_ev"
	    find_callback_record $record_list "id_ev_harmonic" "id_ev_harmonic"
	    find_callback_record $record_list "id_ev_params"   "id_ev_params"
	    find_callback_record $record_list "id_ev_enabled"  "id_ev_enabled"
	}

	if { $IMCAGui::have_filters } {
	    find_callback_record $record_list "filter1"        "filter1"
	    find_callback_record $record_list "filter2"        "filter2"
	}

	start_imcagui_callbacks
}

############################################################################

namespace eval IMCAGui {
	variable program_label

	switch -exact -- $beamline_name {
		imca17id { set program_label "IMCA-CAT Sector 17-ID" }
		imca17bm { set program_label "IMCA-CAT Sector 17-BM" }
		default { set program_label \
				"Unrecognized Beamline '$beamline_name'" }
	}

	if { $beamline_name == "imca17id" } {
		variable minimum_edge_energy  7000
		variable maximum_edge_energy 14500

		variable have_undulator 1

		variable have_filters 1

		variable show_slit_positions 0
	} else {
		variable minimum_edge_energy  5920
		variable maximum_edge_energy 14000

		variable have_undulator 0

		variable have_filters 0

		variable show_slit_positions 0
	}

	# The array edge_array stores information about the available
	# element and edge combinations as read from 'elements.dat'
	#

	variable edge_array

	variable filter1_thickness
	variable filter2_thickness
	variable filter1_position
	variable filter2_position
	variable thickness_list
	variable thickness_indices

	# Foil thickness is in micrometers.

	set foil_thickness 25.0

	set filter1_thickness(0)     0
	set filter1_thickness(1)     1
	set filter1_thickness(2)     7
	set filter1_thickness(3)    16
	set filter1_thickness(4)    25

	set filter1_position(0)      0
	set filter1_position(1)  16800
	set filter1_position(2)  33500
	set filter1_position(3)  50700
	set filter1_position(4)  67450

	set filter2_thickness(0)     0
	set filter2_thickness(1)     2
	set filter2_thickness(2)     4
	set filter2_thickness(3)    14
	set filter2_thickness(4)    40

	set filter2_position(0)      0
	set filter2_position(1)  16900
	set filter2_position(2)  33900
	set filter2_position(3)  50900
	set filter2_position(4)  67775

	set nelements1 [ array size filter1_thickness ]
	set nelements2 [ array size filter2_thickness ]

	# Multiply the number of foil thicknesses by the thickness
	# of an individual foil.

	for { set i 0 } { $i < $nelements1 } { incr i } {
		set filter1_thickness($i) \
			[ expr ( $filter1_thickness($i) * $foil_thickness ) ]
	}
	for { set i 0 } { $i < $nelements2 } { incr i } {
		set filter2_thickness($i) \
			[ expr ( $filter2_thickness($i) * $foil_thickness ) ]
	}

	# Make a list of all the foil thickness combinations.

	for { set i 0 } { $i < $nelements1 } { incr i } {
		for { set j 0 } { $j < $nelements2 } { incr j } {

			set t1 $filter1_thickness($i)
			set t2 $filter2_thickness($j)

			lappend thickness_list [ expr ($t1 + $t2) ]

			lappend thickness_indices [ list $i $j ]
		}
	}
}

############################################################################

proc optimize_menu_handler { frame } {

	# If the menu entries have already been created, just return.

	if [ winfo exists $frame.optimize.tag ] {
		return
	}

	label $frame.optimize.tag	;# Do not pack or grid this.

	set menu $frame.optimize.menu

	$menu add command -label "Manual Optimize..." \
		-command [ code eval exec "optimize &" ]

	$menu add command -label "Auto Optimize Monochromator..." \
		-command [ code eval exec "optimize_auto_mono &" ]

	$menu add command -label "Auto Optimize Slits..." \
		-command [ code eval exec "optimize_auto_slits &" ]

	if { $IMCAGui::beamline_name == "imca17id" } {
		$menu add command -label "Auto Optimize Table..." \
			-command [ code eval exec "optimize_auto_table &" ]
	}

	$menu add command -label "Auto Optimize All..." \
		-command [ code eval exec "optimize_auto_all &" ]

}

############################################################################

proc move_filters { record_list grid } {

	set filter1 [ $record_list get_record "filter1" ]
	set filter2 [ $record_list get_record "filter2" ]

	set thickness_desired [ $grid.filter_desired get ]

	set i [ lsearch -exact $IMCAGui::thickness_list $thickness_desired ]

	if { $i < 0 } {
		set t "Attenuator thickness $thickness_desired not found in "
		append t "filter thickness list."

		tk_dialog .bad_filter_thickness_dialog \
		"Bad Attenuator Thickness" $t error 0 "OK"

		return
	}

	set index_list [ lindex $IMCAGui::thickness_indices $i ]

	set index1 [ lindex $index_list 0 ]
	set index2 [ lindex $index_list 1 ]

	set filter1_position $IMCAGui::filter1_position($index1)
	set filter2_position $IMCAGui::filter2_position($index2)

	set button .main.filter_frame.grid.move

	set_move_state_to_moving $button

	$filter1 move_absolute $filter1_position
	$filter2 move_absolute $filter2_position
}

############################################################################

proc get_filter_thickness {} {

	global callback_values
	global callback_records

	if [ $callback_records(filter1) is_busy ] {
		return "Moving"
	}
	if [ $callback_records(filter2) is_busy ] {
		return "Moving"
	}

	set threshold 10

	set position1 $callback_values(filter1)
	set position2 $callback_values(filter2)

	set thickness1 -1
	set thickness2 -1

	set nelements [ array size IMCAGui::filter1_position ]

	for { set i 0 } { $i < $nelements } { incr i } {

		set item_position $IMCAGui::filter1_position($i)

		set position_diff \
			[ expr ( abs( $item_position - $position1 ) ) ]

		if { $position_diff < $threshold } {
			set thickness1 $IMCAGui::filter1_thickness($i)
			break
		}
	}

	if { $thickness1 < 0 } {
		return "Fault"
	}

	set nelements [ array size IMCAGui::filter2_position ]

	for { set i 0 } { $i < $nelements } { incr i } {

		set item_position $IMCAGui::filter2_position($i)

		set position_diff \
			[ expr ( abs( $item_position - $position2 ) ) ]

		if { $position_diff < $threshold } {
			set thickness2 $IMCAGui::filter2_thickness($i)
			break
		}
	}

	if { $thickness2 < 0 } {
		return "Fault"
	}

	set result [ expr ( $thickness1 + $thickness2 ) ]

	return $result
}

############################################################################

proc create_filter_frame { frame  record_list} {

        set grid $frame.grid

        frame $grid
        pack $grid -side top

        # Create the filter control grid top labels.

	label $grid.frame_label -text "Attenuation:  "
        label $grid.current -text "Current"
        label $grid.desired -text "Desired"

	grid $grid.frame_label  -row 0 -column 0
        grid $grid.current -row 0 -column 1 -columnspan 2
        grid $grid.desired -row 0 -column 3 -columnspan 2

        # Create the filter row.

        label $grid.filter_label -text "Al Thickness "
        label $grid.filter_current -fg blue -text "---"
        label $grid.filter_current_units -text "um  "        

	iwidgets::optionmenu $grid.filter_desired

        set sorted_thickness_list [ lsort -real $IMCAGui::thickness_list ]

        foreach item $sorted_thickness_list {
                $grid.filter_desired insert end $item
        }

#        set thickness [ get_filter_thickness ]
        set thickness 0
        if { $thickness == "Moving" } {
                set thickness 0
        }
        if { $thickness == "Fault" } {
                set thickness 0
        }
        set selection [ lsearch -exact $sorted_thickness_list $thickness ]

        if { $selection < 0 } {
                set selection 0
        }

        $grid.filter_desired select $selection

        button $grid.move -text "Move" -width 6 \
                -command [ code move_filters $record_list $grid ]

	$grid.move configure -disabledforeground \
			[ $grid.move cget -foreground ]

        label $grid.filter_desired_units -text "um  "

        grid $grid.filter_label         -row 1 -column 0
        grid $grid.filter_current       -row 1 -column 1
        grid $grid.filter_current_units -row 1 -column 2 -sticky w
        grid $grid.filter_desired       -row 1 -column 3
        grid $grid.filter_desired_units -row 1 -column 4 -sticky w
        grid $grid.move                 -row 1 -column 5

        # Create the bottom row.

        #set bottom $frame.bottom
        set bottom $frame

        #frame $bottom
        #pack $bottom -side top

#        pack $bottom.move -side right
}

############################################################################

proc update_filter_frame { frame } {
	global callback_values

       	set grid $frame.grid

       	set window $grid.filter_current
	set button $grid.move

       	set thickness [ get_filter_thickness ]

         $window configure -text $thickness

       	if { $thickness == "Fault" } {
               	$window configure -fg red
       	} elseif { $thickness == "Moving" } {
               	$window configure -fg limegreen
       	} elseif { $thickness == "0.0" } {
               	$window configure -fg blue
       	} else {
               	$window configure -fg red
       	}

	set busy_list [ list ]
	lappend busy_list $callback_values(filter1_busy)
	lappend busy_list $callback_values(filter2_busy)

	compute_move_state $button $busy_list "Move"
}

############################################################################

proc create_help_menu { menubar } {

	menu $menubar.help -tearoff 0
	$menubar add cascade -label Help -menu $menubar.help

	$menubar.help add command -label "About IMCA GUI" \
			-command menubar_about_command

	$menubar.help add command -label "Help for IMCA 17-ID" \
			-command [ code display_help "imca17id.html" ]

	$menubar.help add command -label "Help for IMCA 17-BM" \
			-command [ code display_help "imca17bm.html" ]

	$menubar.help add command -label "IMCA FAQ" \
			-command [ code display_help "imca_faq.html" ]

	$menubar.help add command -label "IMCA-CAT FAQ" \
			-command [ code display_help "imca_cat_faq.html" ]

	proc menubar_about_command {} {
		set t "IMCA GUI was written by William Lavender based on "
		append t "a design by the IMCA-CAT staff."

		tk_dialog .about_dialog "About IMCA GUI" $t info 0 "Done"
	}
}

